/* Copyright (c) 2013 Nordic Semiconductor. All Rights Reserved.
 *
 * Use of this source code is governed by a BSD-style license that can be
 * found in the license.txt file.
 */

#include "ble_temperature.h"
#include <string.h>
#include "nordic_common.h"
#include "ble_srv_common.h"
#include "app_util.h"


/**@brief Function for handling the Connect event.
 *
 * @param[in]   p_lbs       LED Button Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_connect(ble_temperature_t * p_temperature, ble_evt_t * p_ble_evt)
{
    p_temperature->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
}


/**@brief Function for handling the Disconnect event.
 *
 * @param[in]   p_lbs       LED Button Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_disconnect(ble_temperature_t * p_temperature, ble_evt_t * p_ble_evt)
{
    UNUSED_PARAMETER(p_ble_evt);
    p_temperature->conn_handle = BLE_CONN_HANDLE_INVALID;
}


/**@brief Function for handling the Write event.
 *
 * @param[in]   p_temperature  Temperature Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_write(ble_temperature_t * p_temperature, ble_evt_t * p_ble_evt)
{
    if (p_temperature->is_notification_supported)
    {
        ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;
        
        if (
            (p_evt_write->handle == p_temperature->temperature_handles.cccd_handle)
            &&
            (p_evt_write->len == 2)
        )
        {
            // CCCD written, call application event handler
            if (p_temperature->evt_handler != NULL)
            {
                ble_temperature_evt_t evt;
                
                if (ble_srv_is_notification_enabled(p_evt_write->data))
                {
                    evt.evt_type = BLE_TEMPERATURE_EVT_NOTIFICATION_ENABLED;
                }
                else
                {
                    evt.evt_type = BLE_TEMPERATURE_EVT_NOTIFICATION_DISABLED;
                }

                p_temperature->evt_handler(p_temperature, &evt);
            }
        }
    }
}


void ble_temperature_on_ble_evt(ble_temperature_t * p_temperature, ble_evt_t * p_ble_evt)
{
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            on_connect(p_temperature, p_ble_evt);
            break;
            
        case BLE_GAP_EVT_DISCONNECTED:
            on_disconnect(p_temperature, p_ble_evt);
            break;
            
        case BLE_GATTS_EVT_WRITE:
            on_write(p_temperature, p_ble_evt);
            break;
            
        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for encoding a Temperature Measurement.
 *
 * @param[in]   p_temperature      Temperature Service structure.
 * @param[in]   temperature        Measurement to be encoded.
 * @param[out]  p_encoded_buffer   Buffer where the encoded data will be written.
 *
 * @return      Size of encoded data.
 */
static uint8_t temperature_encode(ble_temperature_t * p_temperature, uint16_t temperature, uint8_t * p_encoded_buffer)
{
    uint8_t len   = 0;

		p_encoded_buffer[len++] = (uint8_t)(temperature>>8);
		p_encoded_buffer[len++] = (uint8_t)temperature;	
    
    return len;
}


/**@brief Function for adding the XXX temperature characteristic.
 *
 */
static uint32_t temperature_char_add(ble_temperature_t * p_temperature, const ble_temperature_init_t * p_temperature_init)
{

    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
		uint8_t encoded_initial_temperature[2];
/*	
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    //BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
		cccd_md.write_perm = p_temperature_init->temperature_attr_md.cccd_write_perm;
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;
*/	
    if (p_temperature->is_notification_supported)
    {
        memset(&cccd_md, 0, sizeof(cccd_md));
    
        BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
        cccd_md.write_perm = p_temperature_init->temperature_attr_md.cccd_write_perm;
        cccd_md.vloc = BLE_GATTS_VLOC_STACK;
    }
    
    memset(&char_md, 0, sizeof(char_md));
    
    char_md.char_props.read   = 1;
    char_md.char_props.notify = (p_temperature->is_notification_supported) ? 1 : 0;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = (p_temperature->is_notification_supported) ? &cccd_md : NULL;
    char_md.p_sccd_md         = NULL;
    
    ble_uuid.type = p_temperature->uuid_type;
    ble_uuid.uuid = TEMPERATURE_UUID_TEMPERATURE_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    attr_md.read_perm  = p_temperature_init->temperature_attr_md.read_perm;
    attr_md.write_perm = p_temperature_init->temperature_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 1;
		
    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = 2; 
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = 2;
    attr_char_value.p_value      = encoded_initial_temperature;
 	
    return sd_ble_gatts_characteristic_add(p_temperature->service_handle, 
																					&char_md,
																					&attr_char_value,
																					&p_temperature->temperature_handles);
}

uint32_t ble_temperature_init(ble_temperature_t * p_temperature, const ble_temperature_init_t * p_temperature_init)
{
    uint32_t   err_code;
    ble_uuid_t ble_uuid;

    // Initialize service structure
	  p_temperature->evt_handler = p_temperature_init->evt_handler;
    p_temperature->conn_handle = BLE_CONN_HANDLE_INVALID;
    p_temperature->is_notification_supported = p_temperature_init->support_notification;
    
    // Add service
    ble_uuid128_t base_uuid = TEMPERATURE_UUID_BASE;
    err_code = sd_ble_uuid_vs_add(&base_uuid, &p_temperature->uuid_type);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
    
    ble_uuid.type = p_temperature->uuid_type;
    ble_uuid.uuid = TEMPERATURE_UUID_SERVICE;

    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &ble_uuid, &p_temperature->service_handle);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
    
    err_code = temperature_char_add(p_temperature, p_temperature_init);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
    
    return NRF_SUCCESS;
}

uint32_t ble_temperature_measurement_update(ble_temperature_t * p_temperature, uint16_t temperature)
{
    //uint32_t err_code = NRF_SUCCESS;
		uint8_t                encoded_temperature[2] = {0x00,0x00};
		uint16_t               len;
		//uint16_t               hvx_len;
		//ble_gatts_hvx_params_t hvx_params;
		
		len     = temperature_encode(p_temperature, temperature, encoded_temperature);;

    ble_gatts_hvx_params_t params;
    
    memset(&params, 0, sizeof(params));
    params.type = BLE_GATT_HVX_NOTIFICATION;
    params.handle = p_temperature->temperature_handles.value_handle;
    params.p_data = encoded_temperature;
    params.p_len = &len;
    
    return sd_ble_gatts_hvx(p_temperature->conn_handle, &params);
		
/*
		if ((p_temperature->conn_handle != BLE_CONN_HANDLE_INVALID) && p_temperature->is_notification_supported)
		{
				memset(&hvx_params, 0, sizeof(hvx_params));
        
        hvx_params.handle   = p_temperature->temperature_handles.value_handle;
        hvx_params.type     = BLE_GATT_HVX_NOTIFICATION;
        hvx_params.offset   = 0;
        hvx_params.p_len    = &hvx_len;
        hvx_params.p_data   = encoded_temperature;
        
        err_code = sd_ble_gatts_hvx(p_temperature->conn_handle, &hvx_params);		
        if ((err_code == NRF_SUCCESS) && (hvx_len != len))
        {
            err_code = NRF_ERROR_DATA_SIZE;
        }
		}
		else
		{
				err_code = NRF_ERROR_INVALID_STATE;
		}

    return err_code;
*/
}

